package irnatura.offline;

import irnatura.offline.BgFrame;
import irnatura.offline.ImageWrap;
import irnatura.offline.TempProfile;
import irnatura.offline.TempForwardIterator;

public class BgShort extends BgFrame implements TempForwardIterator
{
    public BgShort()
    {
        m_data = null;
        m_origin = 0;
        m_num_samples = 0;
    }

    private short [] m_data; // The deltas

    public int length() { return (m_data == null) ? 0 : m_data.length; }

    // Callback for load
    protected void initStorage(int size)
    {
        if (m_data == null || m_data.length != size)
            m_data = new short[size];
    }

    // Callback for load
    protected void setDelta(int index, short delta)
    {
        m_data[index] = delta;
    }

    // Return number of discarded data points (from old or new).
    public int add(ImageWrap img)
    {
        if (m_data == null) {
            load(img);
            return 0;
        }
        if (m_data.length != img.getSize()) {
            throw new java.lang.ArrayIndexOutOfBoundsException(
                    "Bg length=" + m_data.length +
                    " img size=" + img.getSize());
        }

        for (int i = 0; i < m_data.length; ++i) {
            short newdelta = (short)(img.getTemp(i) - m_origin);
            short curdelta = m_data[i];
            short avgdelta = runningAvg(curdelta, newdelta);
            m_data[i] = avgdelta;
        }
        ++m_num_samples;
        return 0;
    }

    public TempForwardIterator iterator()
    {
        return this;
    }

    //--------------------------------------
    // TempForwardIterator partial interface

    public int getTemp(int x, int y)
    {
        return m_origin + m_data[x + y * m_width];
    }

    public int getTemp(int i)
    {
        return m_origin + m_data[i];
    }

    public int getDelta(int i)
    {
        return m_data[i];
    }
}
