package irnatura.offline;

import irnatura.offline.TempForwardIterator;
import java.util.TreeMap;

public class Histo
{
    private int m_min;
    private int m_max;
    private int [] m_histo;

    public Histo(int count, TempForwardIterator it, int min, int max)
    {
        m_min = min;
        m_max = max;
        m_histo = new int[count];
        float range = (float)(max - min);
        for (int i = 0; i < it.getSize(); ++i) {
            int t = it.getTemp(i);
            int slot = temp2slot(t);
            ++m_histo[slot];
        }
    }

    public int temp2slot(int temp)
    {
        int slot = (m_histo.length * (temp - m_min)) / (m_max - m_min + 1);
        // Sanity
        if (slot < 0) slot = 0;
        if (slot > m_histo.length - 1) slot = m_histo.length - 1;
        return slot;
    }

    public int slot2temp(int slot)
    {
        int temp = m_min + ((slot * (m_max - m_min + 1)) / m_histo.length);
        return temp;
    }

    public String slot2tempstr(int slot)
    {
        return String.format("%.2fC", ((float)slot2temp(slot))/100.0f);
    }

    // Check that less than max_pop pixels are in the top t% of temperatures.
    public String check_topk_temps(int temppct, int max_pop)
    {
        if (temppct >= 100 || max_pop <= 0)
            return "";
        int cnt = 0;
        final int limit = (100 - temppct) * m_histo.length / 100;
        for (int i = m_histo.length - 1; i >= limit; --i) {
            cnt += m_histo[i];
            if (cnt > max_pop)
                return "";
        }
        return "Count(temp > " + slot2tempstr(limit) + ") = " + cnt;
    }

    // Check that the k hottest pixel is at least t% cooler than the max temp.
    // Default pixcnt=64, temppct=16.
    // Seems to falsely trigger frequently when spread is wide.
    public String check_topk_pixels(int pixcnt, int temppct)
    {
        if (pixcnt <= 0 || temppct >= 100)
            return "";
        int max_temp_slot = (100 - temppct) * m_histo.length / 100;
        int cnt = 0;
        for (int i = m_histo.length - 1; i > max_temp_slot; --i) {
            cnt += m_histo[i];
            if (cnt > pixcnt)
                return "";
        }
        return "Pixel #" + cnt + " < " + slot2tempstr(max_temp_slot);
    }

    // 1 = sensitive (more false positives), 2 = less sensitive
    public boolean has_hotspot(int threshold)
    {
        return (check_topk_temps(20, 20 * threshold).equals("")) ||
               (check_topk_pixels(64, 8 * threshold).equals("")) ;
    }

    public int [] getArray() { return m_histo; }

    // Unused, unneeded?
    public static TreeMap<Integer, Integer>
    computeFullHisto(int [] temps)
    {
        // TODO: compare android.util.SparseIntArray
        TreeMap<Integer, Integer> histo = new TreeMap<Integer, Integer>();
        for (int t : temps) {
            Integer cur = histo.get(t);
            int count = (cur == null) ? 0 : cur;
            histo.put(t, count + 1);
        }
        return histo;
    }
}
