package irnatura.offline;

import irnatura.offline.TempProfile;
import irnatura.offline.TempForwardIterator;

public class ImageWrap implements TempForwardIterator
{
    private int m_width;
    private int m_height;
    private int [] m_temps;
    private TempProfile m_tprof;

    public int getWidth() { return m_width; }
    public int getHeight() { return m_height; }
    public int [] getTemps() { return m_temps; }
    public int getSize() { return m_temps.length; }
    public TempProfile getTempProfile() { return m_tprof; }

    public ImageWrap(int [] temps, int width, int height)
    {
        m_width = width;
        m_height = height;
        m_temps = temps;
        if (m_temps == null) {
            m_temps = new int[width * height];
        }
        else if (m_temps.length != m_width * m_height) {
            throw new java.lang.ArrayIndexOutOfBoundsException(
                    "Array length=" + m_temps.length +
                    " expecting " + m_width + "x" + m_height);
        }
        m_tprof = null;
    }

    public int getTemp(int i) { return m_temps[i]; }
    public int getTemp(int x, int y) { return m_temps[x + y * m_width]; }

    // for TempForwardIterator interface, really for BgFrame and unused.
    public int getDelta(int i) { return m_temps[i]; }

    public String getDescription()
    {
        return "";
    }

    // NOTE: currently, should call finishUpdate when done (optimization)
    public void setTemp(int i, int newval)
    {
        int oldval = m_temps[i];
        m_temps[i] = newval;
        if (m_tprof != null) { // must tweak min/max (TODO: mean also)
            m_tprof.tweak(oldval, newval);
        }
    }
    public void setTemp(int x, int y, int newval)
    {
        setTemp(x + y * m_width, newval);
    }
    // Call this after finishing updating temps.
    public void finishUpdate()
    {
        if (m_tprof != null)
            m_tprof.remainingStats();
    }

    public TempProfile getOrComputeTempProfile()
    {
        if (m_tprof != null)
            return m_tprof;
        return m_tprof = new TempProfile(this);
    }

    public void grayValuesToTemps(TempProfile tp)
    {
        m_tprof = tp;
        // Translate gray values into temperatures
        for (int i = 0; i < m_temps.length; ++i)
            m_temps[i] = tp.fromGrayValue(m_temps[i]);
    }

}
