package irnatura.offline;

import irnatura.offline.TempForwardIterator;
import irnatura.offline.ImageWrap;
import irnatura.offline.TempProfile;
import java.io.File;
import java.lang.UnsupportedOperationException;
import ar.com.hjg.pngj.ImageInfo;
import ar.com.hjg.pngj.PngReaderInt;
import ar.com.hjg.pngj.PngWriter;
import ar.com.hjg.pngj.ImageLineInt;
import ar.com.hjg.pngj.chunks.PngChunkTextVar;

public class TaPng
{
    public static ImageWrap readPng(File infile)
    {
        PngReaderInt pr = new PngReaderInt(infile);
        int width = pr.imgInfo.cols;
        int height = pr.imgInfo.rows;
        ImageWrap img = new ImageWrap(null, width, height);
        if (!pr.imgInfo.greyscale)
            throw new UnsupportedOperationException("Input not greyscale");
        if (pr.imgInfo.indexed)
            throw new UnsupportedOperationException("Input indexed/paletted");
        if (pr.imgInfo.alpha)
            throw new UnsupportedOperationException("Input has alpha");

        // Read gray values into img
        for (int h = 0; h < height; ++h) {
            ImageLineInt row = pr.readRowInt();
            int [] line = row.getScanline();
            for (int w = 0; w < width; ++w) {
                img.setTemp(w, h, line[w]);
            }
        }

        pr.end(); // Forces read of remaining metadata.

        // Get Comment metadata (only available after pr.end() above).
        String comm = pr.getMetadata().getTxtForKey(
                PngChunkTextVar.KEY_Comment);
        if (comm.equals(""))
            throw new UnsupportedOperationException("Input no description");

        TempProfile tp = new TempProfile(comm);
        // Translate gray values into temperatures
        img.grayValuesToTemps(tp);

        return img;
    }

    public static void
    writePng(File outfile, TempForwardIterator img, TempProfile tp)
    {
        final boolean alpha = false;
        final boolean grayscale = true;
        final boolean indexed = false;
        ImageInfo ii = new ImageInfo(img.getWidth(), img.getHeight(),
                                     tp.getBitDepth(),
                                     alpha, grayscale, indexed);
        PngWriter pw = new PngWriter(outfile, ii, true);
        pw.getMetadata().setText(PngChunkTextVar.KEY_Source, "ThermApp");
        pw.getMetadata().setText(PngChunkTextVar.KEY_Software,
                                 "ThermApp Lossless Gray");
        // Comment: recoverable TempProfile numbers
        String comm = tp.toString();
        if (comm != null && !comm.equals("")) {
            pw.getMetadata().setText(PngChunkTextVar.KEY_Comment, comm);
        }
        // Description: bg additional info
        String descr = img.getDescription();
        if (descr != null && !descr.equals("")) {
            pw.getMetadata().setText(PngChunkTextVar.KEY_Description, descr);
        }
        ImageLineInt row = new ImageLineInt(ii);
        int [] line = row.getScanline();
        for (int h = 0; h < img.getHeight(); ++h) {
            for (int w = 0; w < img.getWidth(); ++w) {
                line[w] = tp.getGrayValue(img.getTemp(w, h));
            }
            pw.writeRow(row);
        }
        pw.end();
    }
}
