package irnatura.online;

// BGMODE: probably obsolete support for background modes.

import irnatura.offline.BgFrame;
import irnatura.offline.BgShort;
import irnatura.offline.Histo;
import irnatura.offline.ImageWrap;
import irnatura.offline.TaPng;
import irnatura.offline.TempProfile;

import thermapp.sdk.sample.R;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;

import android.media.MediaScannerConnection;
import android.media.MediaScannerConnection.MediaScannerConnectionClient;
 
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.Matrix;
//import android.media.Ringtone;
import android.media.RingtoneManager;
import android.media.MediaPlayer;
import android.net.Uri;
import android.os.BatteryManager;
import android.view.Display;
import android.view.KeyEvent;
import android.view.OrientationEventListener;
import android.view.Surface;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;
 

// SY: to make file visible to Windows Explorer.
// May want to keep instance (of msc) instead of new'ing one each time.
class MyMediaScanner implements MediaScannerConnectionClient
{
    private MediaScannerConnection m_msc;
    private String m_path;

    MyMediaScanner(Context ctxt, String fname) {
        m_path = fname;
        m_msc = new MediaScannerConnection(ctxt, this);
        m_msc.connect();
    }

    @Override
    public void onMediaScannerConnected()
    {
        m_msc.scanFile(m_path, null);
    }

    @Override
    public void onScanCompleted(String path, Uri uri)
    {
        m_msc.disconnect();
    }
}
 
// Call 'ding' in two modes:
// - normal mode: play Android default 'notify' sound, clean up on completion.
// - 'narrow' mode: play notify.wav, on completion call
//   m_narrow_cback.onNarrowDingComplete() to reset 'WildState.m_dingmode' bit.
class Dinger implements MediaPlayer.OnCompletionListener
{
    private final Activity m_main_act;
    private final WildState m_narrow_cback;

    public Dinger(Activity main_act, WildState narrow_cback)
    {
        m_main_act = main_act;
        m_narrow_cback = narrow_cback;
    }

    private Uri m_alert_uri = null;
    private MediaPlayer m_media_player = null;
    private boolean m_do_narrow_cback = false;

    public String ding(boolean narrow)
    {
        m_do_narrow_cback = narrow;
        if (m_media_player != null) // Already dinging
            return null;

        Context ctxt = m_main_act.getApplicationContext();
        if (narrow) {
            // Copied from C:\Windows\Media\notify.wav
            m_media_player = MediaPlayer.create(ctxt, R.raw.notify);
        }
        else {
            if (m_alert_uri == null) {
                m_alert_uri = RingtoneManager.getDefaultUri(
                                RingtoneManager.TYPE_NOTIFICATION);
                if (m_alert_uri == null)
                    return "Audio URI is null";
            }
            //// A: may interrupt other sounds
            //Ringtone r = RingtoneManager.getRingtone(ctxt, m_alert_uri);
            //r.play();

            // B: prevents interrupting other sounds
            m_media_player = MediaPlayer.create(ctxt, m_alert_uri);
        }
        m_media_player.setOnCompletionListener(this);
        m_media_player.start();

        return null;
    }

    public void onCompletion(MediaPlayer mp)
    {
        if (m_media_player != null) {
            m_media_player.release();
            m_media_player = null;
        }
        if (m_do_narrow_cback) {
            m_narrow_cback.onNarrowDingComplete();
        }
    }
}

class MyOrientationEventListener extends OrientationEventListener
{
    private Button m_btn_pic;
    private int m_last_orient = ORIENTATION_UNKNOWN;

    public MyOrientationEventListener(Context context, Button btn)
    {
        super(context);
        m_btn_pic = btn;
    }

    public void onOrientationChanged(final int orientation)
    {
        if (orientation == ORIENTATION_UNKNOWN)
            return;
        int orient = ((orientation + 45) / 90) % 4;
        if (m_last_orient != orient) {
            m_last_orient = orient;
            String picdir = "PIC";
            switch (orient) {
                case 0: picdir = "PiC"; break;
                case 1: picdir = ":PIC"; break;
                case 2: picdir = "P!C"; break;
                case 3: picdir = "PIC:"; break;
            }
            final String btnlabel = picdir;
            m_btn_pic.post(new Runnable() {
                public void run() {
                    m_btn_pic.setText(btnlabel);
                }
                });
        }
    }

    public Bitmap rotate(Bitmap bmp, int base_rotation)
    {
        int deg = base_rotation;
        switch (m_last_orient) {
            case 0: deg += 0; break;
            case 1: deg += 90; break;
            case 2: deg += 180; break;
            case 3: deg += 270; break;
        }
        deg = deg % 360;
        if (deg == 0)
            return bmp;
        Matrix m = new Matrix();
        m.postRotate(deg);
        return Bitmap.createBitmap(bmp, 0, 0,
                                   bmp.getWidth(), bmp.getHeight(),
                                   m, true);
    }
}

public class WildState
{
    private TextView tv_min = null;
    private TextView tv_max = null;
    private TextView tv_status = null;
 
    private boolean m_celsius = true;

    private Activity m_main_act;
    private SharedPreferences m_prefs;
    private String m_media_path;

    public WildState(Activity main_act,
                     SharedPreferences prefs,
                     String media_path)
    {
        m_main_act = main_act;
        m_prefs = prefs;
        m_media_path = media_path;
    }
 
    private String Temp2Str(int c100)
    {
        int c = Math.round((float)c100 / 100.0f);
        int f = Math.round((float)c100 * 0.018f + 32.0f);
        return m_celsius ? (c + " C (" + f + " F)") : (f + " F (" + c + " C)");
    }

    private BgFrame m_bgframe = null;

    private BgFrame getBgFrame()
    {
        if (m_bgframe == null) {
            //m_bgframe = new BgByte();
            m_bgframe = new BgShort();
        }
        return m_bgframe;
    }

    private boolean getOrLoadBgFrame(int expected_length)
    {
        if (getBgFrame().isLoaded()) {
            if (getBgFrame().length() == expected_length)
                return true;
            statusQueue("BG len " + getBgFrame().length()
                        + "!=" + expected_length);
        }
        try {
            final String filename = "bg.t.png";
            ImageWrap img = TaPng.readPng(new File(m_media_path, filename));
            getBgFrame().load(img);
            if (getBgFrame().length() == expected_length) {
                statusQueue("Loaded bg " + filename);
                return true;
            }
            statusQueue(filename + " len "
                        + getBgFrame().length() + "!=" + expected_length);
        }
        catch (Exception e) {
            statusQueue(e.getMessage());
        }
        return false;
    }
 
    private String getBatteryLevel()
    {
        IntentFilter ifilter = new IntentFilter(Intent.ACTION_BATTERY_CHANGED);
        Intent batteryStatus = m_main_act.registerReceiver(null, ifilter);
        int level = batteryStatus.getIntExtra(BatteryManager.EXTRA_LEVEL, -1);
        int scale = batteryStatus.getIntExtra(BatteryManager.EXTRA_SCALE, -1);
        return " (battery " + ((level * 100)/scale) + "%)";
    }

    // FIXME: doesn't work (always 0)
    // TODO: Try OrientationEventListener (1-degree granularity)
    private int getCurrentRotation()
    {
        WindowManager wm = m_main_act.getWindowManager();
        //WindowManager wm = (WindowManager)
        //              getSystemService(Context.WINDOW_SERVICE);
        Display d = wm.getDefaultDisplay();
        switch (d.getRotation()) {
            case Surface.ROTATION_0:
                return 0;
            case Surface.ROTATION_90:
                return 90;
            case Surface.ROTATION_180:
                return 180;
            case Surface.ROTATION_270:
                return 270;
            default:
                return 0;
        }
    }

    public static void wrapPostBackgroundColor(final View v, final int c)
    {
        v.post(new Runnable() {
            public void run() {
                v.setBackgroundColor(c);
            }
        });
    }
 
    public static void wrapPost(final TextView tv, final String s)
    {
        tv.post(new Runnable() {
            public void run() {
                tv.setText(s);
            }
        });
    }
 
    private String m_status;
    public void statusQueue(final String s)
    {
        if (m_status == null)
            m_status = s;
        else
            m_status += "; " + s;
    }

    private SimpleDateFormat m_sdf;
    private GregorianCalendar m_next_update;

    // Post a message for 5 seconds.
    public void statusPostMessage(final String s)
    {
        wrapPost(tv_status, s);

        m_next_update = new GregorianCalendar();
        m_next_update.add(Calendar.SECOND, 5);
    }

    public void statusPostQueued()
    {
        if (btn_take_bg_count != 0) // no updates in this mode
            return;
        if (m_status == null) {
            GregorianCalendar now = new GregorianCalendar();
            if (m_next_update != null && m_next_update.after(now)) {
                return;
            }
            if (m_sdf == null)
                m_sdf = new SimpleDateFormat("hh:mm a");
            final String s = m_sdf.format(now.getTime()) + getBatteryLevel();
            wrapPost(tv_status, s);

            m_next_update = now;
            m_next_update.add(Calendar.MINUTE, 1);
            m_next_update.set(Calendar.SECOND, 0);
        }
        else {
            statusPostMessage(m_status);
            m_status = null;
        }
    }

    // Post a fixed message
    private void statusFixedMessage(String msg, boolean is_ui)
    {
        if (is_ui) tv_status.setText(msg);
        else wrapPost(tv_status, msg);
        m_next_update = null;
    }

    private void writeToPng(ImageWrap img, String filename)
    {
        try {
            File file = new File(m_media_path, filename);
            TempProfile tp = img.getOrComputeTempProfile();
            TaPng.writePng(file, img, tp);
            new MyMediaScanner(m_main_act, file.getAbsolutePath());
            statusQueue("Saved " + filename);
        }
        catch (Exception e) {
            final String f_msg = e.getMessage();
            statusQueue("Error " + f_msg);
        }
    }

    public void writeJpg(Bitmap bm, File outfile)
    {
        try {
            OutputStream os = new BufferedOutputStream(
                              new FileOutputStream(outfile));
            if (bm.compress(Bitmap.CompressFormat.JPEG, 90, os)) {
                new MyMediaScanner(m_main_act, outfile.getAbsolutePath());
                statusQueue("Saved " + outfile.getName());
            }
            else
                statusQueue("Compression error");
        }
        catch (java.io.FileNotFoundException e) {
            statusQueue(e.getMessage());
        }
    }

    private Dinger m_dinger = null;

    private void ding(boolean narrow)
    {
        if (m_dinger == null)
            m_dinger = new Dinger(m_main_act, this);
        try {
            String err = m_dinger.ding(narrow);
            if (err != null)
                statusQueue(err);
        }
        catch (Exception e) {
            statusQueue("Ding Exception " + e.getMessage());
        }
    }

    // Callback after m_dinger plays "narrow" sound.
    public void onNarrowDingComplete()
    {
        m_dingmode = true;
        updateHSbutton(false);
    }

    public void onStop()
    {
        if (m_dinger != null) {
            m_dinger.onCompletion(null); // releases MediaPlayer
            m_dinger = null;
        }
    }

    Matrix m_rotate_matrix;

    String m_save_jpg;
    // Return true to bypass default posting.
	public boolean OnFrameGetThermAppBMP(final ImageView iv, Bitmap bmp)
    {
        // Temporary: just emit jpg version of TA-rendered bitmap
        if (m_save_jpg != null) {
            File file = new File(m_media_path, m_save_jpg);
            m_save_jpg = null;
            writeJpg(m_orienter.rotate(bmp, m_rotation), file);
        }
        final Bitmap fbmp = (m_rotate_matrix == null)
                            ? bmp : Bitmap.createBitmap(bmp, 0, 0,
                                        bmp.getWidth(), bmp.getHeight(),
                                        m_rotate_matrix, true);
        iv.post(new Runnable() {
            public void run() { iv.setImageBitmap(fbmp); }
            });
        statusPostQueued();
        return true;
	}

    public void OnFrameGetThermAppTemperatures(final int[] frame,
                                               final int w, final int h)
    {
        // First: check for deferred single-click detection.
        {
            GregorianCalendar now = new GregorianCalendar();
            GregorianCalendar to = m_volup_timeout;
            if (to != null && now.after(to)) {
                // Single-click volume-up action
                onPressToggleBg(false);
                m_volup_timeout = null;
            }
            to = m_voldown_timeout;
            if (to != null && now.after(to)) {
                // Single-click volume-down action
                onPressHS(false);
                m_voldown_timeout = null;
            }
        }

        // Merge background
        boolean merged_bg = false;
        ImageWrap img = new ImageWrap(frame, w, h);
        if (btn_merge_bg
            && btn_take_bg_count == 0
            && getOrLoadBgFrame(frame.length))
        {
            m_bgframe.applyTo(img);
            merged_bg = true;
        }

        TempProfile tp = img.getOrComputeTempProfile();

        wrapPost(tv_min, Temp2Str(tp.getMin()));
        wrapPost(tv_max, Temp2Str(tp.getMax()));

        if (btn_take_pic_pressed) {
            btn_take_pic_pressed = false;
            if (btn_take_bg_count < 0) {
                btn_take_bg_count = -btn_take_bg_count;
                BgFrame bg = getBgFrame();
                int num_overflow = bg.load(img);
                if (num_overflow > 0) {
                    statusQueue("" + num_overflow + " big diffs");
                }
                // NOTE: only writing last bg frame to file. TODO: save merged.
                if (--btn_take_bg_count == 0) {
                    bg.setBadBitMode(btn_take_bg_mode, 2.5f); // BGMODE
                    btn_take_bg_mode = BgFrame.MODE_NULL; // BGMODE
                    writeToPng(img, "bg.t.png");
                }
            }
            else {
                final int i = m_prefs.getInt("fnameCounter", 0) + 1;
                m_prefs.edit().putInt("fnameCounter", i).apply();
                final String fileprefix = "ta" + i + (merged_bg ? "b" : "");
                m_save_jpg = fileprefix + ".jpg";
                if (m_save_llpng) {
                    final String filename = fileprefix + ".t.png";
                    writeToPng(img, filename);
                }
            }
        }
        else if (btn_take_bg_count > 0) {
            int num_discarded = getBgFrame().add(img);
            if (num_discarded > 0) {
                statusQueue(" -" + num_discarded);
            }
            // NOTE: only writing last bg frame to file. TODO: save merged.
            if (--btn_take_bg_count == 0) {
                getBgFrame().setBadBitMode(btn_take_bg_mode, 2.5f); // BGMODE
                btn_take_bg_mode = BgFrame.MODE_NULL; // BGMODE
                writeToPng(img, "bg.t.png");
            }
        }
        else if (m_dingmode) {
            Histo histo = new Histo(256, img, tp.getMin(), tp.getMax());
            boolean spread_too_narrow = m_hot_min_temp_spread != 0
                    && (tp.getMax() - tp.getMin()) < m_hot_min_temp_spread;
            String hss =
                    histo.check_topk_temps(m_hot_temp_pct, m_hot_temp_max_pop);
            String hss2 = "";
            if (!spread_too_narrow)
                hss2 = histo.check_topk_pixels(m_hot_pxl_cnt, m_hot_pxl_cool);
            if (!hss2.equals("")) {
                if (!hss.equals("")) hss += "; ";
                hss += hss2;
            }
            if (hss.equals("")) {
                m_hsframes = 0;
            }
            else {
                if (++m_hsframes >= m_hot_min_frames) {
                    m_dingmode = false;
                    ding(spread_too_narrow);
                    if (spread_too_narrow) {
                        hss = "(narrow)" + hss;
                    }
                    statusPostMessage(hss);
                    updateHSbutton(false);
                }
            }
        }
    }

    public void onResume_PaletteMode()
    {
            tv_min.setVisibility(View.VISIBLE);
            tv_max.setVisibility(View.VISIBLE);
            tv_status.setVisibility(View.VISIBLE);
    }

    private boolean m_dingmode;
    private int m_hsframes;
    private boolean m_save_llpng;
    private int m_rotation;

    private int m_hot_temp_pct = 20;
    private int m_hot_temp_max_pop = 40;
    private int m_hot_pxl_cnt = 64;
    private int m_hot_pxl_cool = 16;
    private int m_hot_min_frames = 2;
    private int m_hot_min_temp_spread = 250;

    private void setDefaultHotParams()
    {
        m_hot_temp_pct = 20;
        m_hot_temp_max_pop = 40;
        m_hot_pxl_cnt = 64;
        m_hot_pxl_cool = 16;
        m_hot_min_frames = 2;
        m_hot_min_temp_spread = 250;
        statusPostMessage("Hot Params Reset (non-persistent)");
    }

    MyOrientationEventListener m_orienter;

    public void onResumeCheckPrefs(SharedPreferences prefs)
    {
        m_celsius = prefs.getString("listUnits", "C").equals("C");
        updateHSbutton(false);
        m_hsframes = 0;
        m_save_llpng = prefs.getBoolean("checkSaveLLPNG", true);

        String hot_temp_params = prefs.getString("hot_temp_params", null);
        if (hot_temp_params != null) {
            String [] spl = hot_temp_params.split("[.]", 2);
            if (spl.length >= 1)
                m_hot_temp_pct = Integer.parseInt(spl[0]);
            if (spl.length >= 2)
                m_hot_temp_max_pop = Integer.parseInt(spl[1]);
        }
        String hot_pixel_params = prefs.getString("hot_pixel_params", null);
        if (hot_pixel_params != null) {
            String [] spl = hot_pixel_params.split("[.]", 2);
            if (spl.length >= 1)
                m_hot_pxl_cnt = Integer.parseInt(spl[0]);
            if (spl.length >= 2)
                m_hot_pxl_cool = Integer.parseInt(spl[1]);
        }
        m_hot_min_frames = Integer.parseInt(
                prefs.getString("hot_min_frames", "2"));

        m_hot_min_temp_spread = Integer.parseInt(
                prefs.getString("hot_min_temp_spread", "250"));

        setRotation(Integer.parseInt(prefs.getString("rotation", "90")));

        m_orienter.enable();
    }

	public void onPause()
    {
        m_orienter.disable();
    }

    private void setRotation(int rotation)
    {
        if (m_rotation == rotation)
            return;
        m_rotation = rotation;
        if (m_rotation == 0) {
            m_rotate_matrix = null;
            return;
        }
        m_rotate_matrix = new Matrix();
        m_rotate_matrix.postRotate(m_rotation);
    }

    // 0: disabled
    // <0: button pressed; pending
    // >0: collecting frames, saving first to file
    private int btn_take_bg_mode = BgFrame.MODE_NULL; // BGMODE
    private int btn_take_bg_count = 0;
    private boolean btn_merge_bg = false;
    private boolean btn_take_pic_pressed = false;
    Button m_btn_hot = null;
    Button m_btn_bg = null;

    // NOTE: this is probably obsolete.
    private static final int [] BG_MODES = { // BGMODE
        0,
        BgFrame.MODE_AVG,
      //BgFrame.MODE_NEIGHBOR,
      //BgFrame.MODE_IMGDEV,
      //BgFrame.MODE_PIXDEV
    };

    private int cycleBgMode(int mode) // BGMODE
    {
        for (int i = 0; i < BG_MODES.length - 1; ++i) {
            if (mode == BG_MODES[i]) {
                return BG_MODES[i+1];
            }
        }
        return 0;
    }

    private String getTakeBgMessage()
    {
        return "Press PIC to take " + (-btn_take_bg_count) + " BG"
        //   + " (mode " + BgFrame.modestr(btn_take_bg_mode) + ")" // BGMODE
             ;
    }

    private void onPressTakeBg()
    {
        btn_take_bg_mode = cycleBgMode(btn_take_bg_mode); // BGMODE
        if (btn_take_bg_mode != BgFrame.MODE_NULL) {
            if (btn_take_bg_count == 0) {
                btn_take_bg_count = -1;
            }
            statusFixedMessage(getTakeBgMessage(), true);
        }
        else {
            btn_take_bg_count = 0;
            statusPostMessage("Cancelled");
        }
    }

    // is_ui indicates whether this is a UI thread (TODO: must be some other
    // way to detect this automatically).  Since only UI threads can update
    // the button text, we must wrap the update when !is_ui.
    private void onPressHS(boolean is_ui)
    {
        m_dingmode = !m_dingmode;
        m_hsframes = 0;
        updateHSbutton(is_ui);
        statusPostMessage(
            "Temp(" + m_hot_temp_pct + "," + m_hot_temp_max_pop
            + "); Pixel(" + m_hot_pxl_cnt + "," + m_hot_pxl_cool
            + "); #fr=" + m_hot_min_frames
            + " spread=" + m_hot_min_temp_spread
            );
    }

    private void updateHSbutton(boolean is_ui)
    {
        if (m_btn_hot == null)
            return;
        int color = m_dingmode ? 0xffff8888 : 0xffcccccc;
        if (is_ui)
            m_btn_hot.setBackgroundColor(color);
        else
            wrapPostBackgroundColor(m_btn_hot, color);
    }

    // is_ui indicates whether this is a UI thread (TODO: must be some other
    // way to detect this automatically).  Since only UI threads can update
    // the button text, we must wrap the update when !is_ui.
    private void onPressToggleBg(boolean is_ui)
    {
        if (btn_take_bg_count < 0) {
            --btn_take_bg_count;
            statusFixedMessage(getTakeBgMessage(), is_ui);
        }
        else {
            btn_merge_bg = !btn_merge_bg;
            int color = btn_merge_bg ? 0xff8888ff : 0xffcccccc;
            if (is_ui)
                m_btn_bg.setBackgroundColor(color);
            else
                wrapPostBackgroundColor(m_btn_bg, color);
        }
    }

    private void onPressTakePic()
    {
        btn_take_pic_pressed = true;
    }

    public void addListenerOnButtons()
    {
        tv_min = (TextView) m_main_act.findViewById(R.id.TextView_min);
        tv_max = (TextView) m_main_act.findViewById(R.id.TextView_max);
        tv_status = (TextView) m_main_act.findViewById(R.id.TextView_status);
 
        m_btn_hot = (Button) m_main_act.findViewById(R.id.btn_hot);
        m_btn_hot.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) { onPressHS(true); }
        });
        m_btn_hot.setOnLongClickListener(new View.OnLongClickListener() {
            @Override
            public boolean onLongClick(View v) {
                setDefaultHotParams();
                return true;
            }
        });

        Button btn_pic = (Button) m_main_act.findViewById(R.id.btn_pic);
        btn_pic.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) { onPressTakePic(); }
        });

        if (m_orienter == null) {
            m_orienter = new MyOrientationEventListener(m_main_act, btn_pic);
        }

        m_btn_bg = (Button) m_main_act.findViewById(R.id.btn_bg);
        m_btn_bg.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) { onPressToggleBg(true); }
        });
        m_btn_bg.setOnLongClickListener(new View.OnLongClickListener() {
            @Override
            public boolean onLongClick(View v) {
                onPressTakeBg();
                return true;
            }
        });
    }

    // For long-click detection
    private boolean m_voldown_long = false;
    private boolean m_volup_long = false;

    // For double-click detection
    GregorianCalendar m_volup_timeout;
    GregorianCalendar m_voldown_timeout;

    // SY: Observationally, a long press triggers:
    //  KeyDown, KeyLongPress, KeyDown, KeyUp
    public boolean onKeyDown(int keyCode, KeyEvent event)
    {
        if (keyCode == KeyEvent.KEYCODE_VOLUME_UP ||
            keyCode == KeyEvent.KEYCODE_VOLUME_DOWN)
        {
            event.startTracking();
            return true;
        }
        return false;
    }

    public boolean onKeyUp(int keyCode, KeyEvent event)
    {
        if (keyCode == KeyEvent.KEYCODE_VOLUME_UP) {
            if (m_volup_long) {
                onPressTakePic();
                m_volup_long = false;
                return true;
            }
            else {
                // Check double click
                GregorianCalendar now = new GregorianCalendar();
                GregorianCalendar to = m_volup_timeout;
                if (to != null && to.after(now)) {
                    // Double-click action
                    onPressTakeBg();
                    m_volup_timeout = null;
                }
                else {
                    // First of possible double-click.
                    // Single-click action activated in
                    // OnFrameGetThermAppTemperatures.
                    now.add(Calendar.MILLISECOND, 600);
                    m_volup_timeout = now;
                }
            }
        }
        if (keyCode == KeyEvent.KEYCODE_VOLUME_DOWN) {
            if (m_voldown_long) {
                onPressTakePic();
                m_voldown_long = false;
                return true;
            }
            else {
                // Check double click
                GregorianCalendar now = new GregorianCalendar();
                GregorianCalendar to = m_voldown_timeout;
                if (to != null && to.after(now))
                {
                    // Double-click action
                    onPressTakeBg();
                    m_voldown_timeout = null;
                }
                else {
                    // First of possible double-click.
                    // Single-click action activated in
                    // OnFrameGetThermAppTemperatures.
                    now.add(Calendar.MILLISECOND, 600);
                    m_voldown_timeout = now;
                }
            }
        }
        return true;
    }

    public boolean onKeyLongPress(int keyCode, KeyEvent event)
    {
        if (keyCode == KeyEvent.KEYCODE_VOLUME_UP) {
            tv_status.setText("Release to Take Photo");
            m_volup_long = true;
            return true;
        }
        if (keyCode == KeyEvent.KEYCODE_VOLUME_DOWN) {
            tv_status.setText("Release to Take Photo");
            m_voldown_long = true;
            return true;
        }
        return false;
    }
}
